"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.SCHEMA = exports.RULE_NAME = exports.MESSAGE_ID = void 0;
var utils_1 = require("@angular-eslint/utils");
var create_rule_1 = require("../utils/create-rule");
var elements_1 = require("../utils/elements");
exports.MESSAGE_ID = 'requireTestId';
exports.RULE_NAME = 'require-test-id';
exports.SCHEMA = [
    {
        type: 'object',
        properties: {
            overrideElements: {
                type: "array",
                items: {
                    type: "string"
                }
            },
            addElements: {
                type: "array",
                items: {
                    type: "string"
                }
            },
            attribute: {
                type: "string"
            },
        },
        additionalProperties: false,
    },
];
var defaultOptions = [
    {
        overrideElements: __spreadArray(__spreadArray([], elements_1.nativeElements, true), elements_1.materialElements, true),
        attribute: 'data-test',
    },
];
exports.default = (0, create_rule_1.createRule)({
    name: exports.RULE_NAME,
    meta: {
        type: 'suggestion',
        docs: {
            description: 'Ensures that elements have test ids',
            recommended: false,
        },
        schema: exports.SCHEMA,
        messages: (_a = {},
            _a[exports.MESSAGE_ID] = 'Element {{element}} should have attribute {{attribute}}',
            _a),
    },
    defaultOptions: defaultOptions,
    create: function (context) {
        var _a;
        var _b;
        var options = [
            __assign(__assign({}, defaultOptions[0]), context.options[0])
        ];
        var elements = ((_b = options[0].addElements) === null || _b === void 0 ? void 0 : _b.length)
            ? __spreadArray(__spreadArray([], options[0].overrideElements || [], true), options[0].addElements, true) : options[0].overrideElements;
        var selector = makeSelector(elements || []);
        var parserServices = (0, utils_1.getTemplateParserServices)(context);
        return _a = {},
            _a[selector] = function (element) {
                if (isAttributePresentInElement(element, options[0].attribute))
                    return;
                context.report({
                    loc: parserServices.convertNodeSourceSpanToLoc(element.sourceSpan),
                    messageId: exports.MESSAGE_ID,
                    data: {
                        element: element.name,
                        attribute: options[0].attribute,
                    },
                });
            },
            _a;
    },
});
function makeSelector(elements) {
    return elements.map(function (element) { return "Element$1[name=".concat(element, "]"); }).join(', ');
}
function isAttributePresentInElement(element, attribute) {
    return element.inputs.some(function (input) { return input.name === attribute; })
        || element.attributes.some(function (attr) { return attr.name === attribute; });
}
